use std::str;

/// parse an string of 16 bytes hex-encoded and return an array of 16 bytes
/// # Examples
///
/// ```no_run
/// let text = String::from("2b7e151628aed2a6abf7158809cf4f3c");
/// let array: [u8; 16] = parse_hex_string(&text);
/// ```
pub fn parse_hex_string(input: &String) -> [u8; 16] {
    let mut result = [0u8; 16];
    // Iterate over the string 2 characters each time
    for (index, byte) in input.as_bytes().chunks(2).enumerate() {
        if let Ok(byte_str) = str::from_utf8(byte) {
            if let Ok(byte_value) = u8::from_str_radix(byte_str, 16) {
                result[index] = byte_value;
            } else {
                panic!("Failed to parse byte");
            }
        } else {
            panic!("Invalid byte string");
        }
    }
    result
}

/// print bytes in an array as an hex-encoded string
pub fn print_array(&array: &[u8; 16]) {
        for &byte in &array {
            print!("{:02x}", byte);
        }
        println!();
    }

/// split any string into an array of 16 bytes, (any additional byte in the string is lost)
pub fn string_to_u8_16(input: &String) -> [u8; 16] {
    let mut result = [0u8; 16];
    let bytes = input.as_bytes();
    let len = bytes.len().min(16);
    result[..len].copy_from_slice(&bytes[..len]);
    result
}

// pub fn u8_16_to_string(input: &[u8; 16]) -> String {
//     let slice = &input[..];
//     let string = str::from_utf8(slice).expect("Invalid UTF-8");
//     string.to_string()
// }

#[cfg(test)]
mod tests {
    use super::*;
    #[test]
    fn parse_hex_string_test() {
        let result: [u8; 16] = parse_hex_string(&String::from("2b7e151628aed2a6abf7158809cf4f3c"));
        let expected: [u8; 16] = [
            0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf,
            0x4f, 0x3c,
        ];
        assert_eq!(result, expected);
    }
}
